@@ -159,7 +159,7 @@ menu_item_cb (gpointer callback_data, guint callback_action, GtkWidget *widget)
 	g_message ("ItemFactory: activated \"%s\"",
 		   gtk_item_factory_path_from_widget (widget));
 
-	dia_canvas_push_undo (view->canvas, NULL);
+	//dia_canvas_push_undo (view->canvas, NULL);
 
 	switch (callback_action) {
 	case FILE_NEW: {
@@ -226,10 +226,11 @@ menu_item_cb (gpointer callback_data, guint callback_action, GtkWidget *widget)
 		gtk_main_quit ();
 		break;
 	case EDIT_UNDO:
-		dia_canvas_pop_undo (view->canvas);
+		dia_undo_manager_undo_transaction (dia_canvas_get_undo_manager (view->canvas));
 		break;
 	case EDIT_REDO:
-		dia_canvas_pop_redo (view->canvas);
+		dia_undo_manager_redo_transaction (dia_canvas_get_undo_manager (view->canvas));
+		//dia_canvas_pop_redo (view->canvas);
 		break;
 	case EDIT_SELECT_ALL: {
 		dia_canvas_view_select_all (view);
@@ -238,11 +239,15 @@ menu_item_cb (gpointer callback_data, guint callback_action, GtkWidget *widget)
 	case EDIT_DEL_FOCUSED:
 		if (view->focus_item) {
 			DiaCanvasItem *item = view->focus_item->item;
-			if (item->parent)
+			if (item->parent) {
+				dia_undo_manager_begin_transaction (dia_canvas_get_undo_manager (view->canvas));
 				dia_canvas_groupable_remove (DIA_CANVAS_GROUPABLE (item->parent), item);
+				dia_undo_manager_commit_transaction (dia_canvas_get_undo_manager (view->canvas));
+			}
 		}
 		break;
 	case EDIT_DEL_SELECTED:
+		dia_undo_manager_begin_transaction (dia_canvas_get_undo_manager (view->canvas));
 		while (view->selected_items) {
 			DiaCanvasItem *item;
 			item = DIA_CANVAS_VIEW_ITEM (view->selected_items->data)->item;
@@ -250,6 +255,7 @@ menu_item_cb (gpointer callback_data, guint callback_action, GtkWidget *widget)
 				dia_canvas_groupable_remove (DIA_CANVAS_GROUPABLE (item->parent), item);
 			}
 		}
+		dia_undo_manager_commit_transaction (dia_canvas_get_undo_manager (view->canvas));
 		break;
 	case EDIT_PROPERTIES: {
 		DiaCanvas *c = dia_canvas_new();
@@ -343,6 +349,7 @@ menu_item_cb (gpointer callback_data, guint callback_action, GtkWidget *widget)
 	}
 }
 
+/*
 static gint
 canvas_item_event(DiaCanvasItem *item, DiaEvent *event)
 {
@@ -355,7 +362,7 @@ canvas_item_event(DiaCanvasItem *item, DiaEvent *event)
 	}
 	return FALSE;
 }
-
+*/
 void
 display_window_destroy (GtkObject *obj, gpointer data)
 {
@@ -431,7 +438,7 @@ add_line_random (DiaCanvasView *view)
 	DiaCanvasViewItem *vitem;
 	guint i;
 
-	dia_canvas_push_undo (view->canvas, NULL);
+	//dia_canvas_push_undo (view->canvas, NULL);
 	item = dia_canvas_item_create (DIA_TYPE_CANVAS_LINE,
 			"color", DIA_COLOR_A (g_random_int_range (0, 200),
 					      g_random_int_range (0, 200),
@@ -470,7 +477,7 @@ add_box_random (DiaCanvasView *view)
 	DiaCanvasItem *item;
 	DiaCanvasViewItem *vitem;
 	
-	dia_canvas_push_undo (view->canvas, NULL);
+	//dia_canvas_push_undo (view->canvas, NULL);
 	item = dia_canvas_item_create (DIA_TYPE_CANVAS_BOX,
 			     "width", g_random_double_range (25.0, 150.0),
 			     "height", g_random_double_range (25.0, 150.0),
@@ -789,7 +796,7 @@ main (int argc, char* argv[])
 
 	dia_canvas_item_rotate (item, 15.0);
 	dia_canvas_item_move (item, 50.0, 50.0);
-	g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
+	//g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
 	/* Box is user later to connect a line to it. */
 	box = item;
 	g_object_unref (item);
@@ -805,7 +812,7 @@ main (int argc, char* argv[])
 	g_object_weak_ref (G_OBJECT (item), item_destroyed, NULL);
 
 	dia_canvas_item_move (item, 50.0, 50.0);
-	g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
+	//g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
 	g_object_unref (item);
 	g_assert (G_OBJECT (item)->ref_count == 1);
 
@@ -824,7 +831,7 @@ main (int argc, char* argv[])
 	g_object_weak_ref (G_OBJECT (item), item_destroyed, NULL);
 
 	dia_canvas_item_move (item, 30.0, 0.0);
-	g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
+	//g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
 	//dia_canvas_item_flip (item, TRUE, FALSE);
 	//dia_canvas_item_rotate (item, 35.0);
 	g_object_unref (item);
@@ -849,7 +856,7 @@ main (int argc, char* argv[])
 	dia_canvas_groupable_add (DIA_CANVAS_GROUPABLE (canvas->root), item);
 	g_object_weak_ref (G_OBJECT (item), item_destroyed, NULL);
 
-	g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
+	//g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
 	dia_canvas_item_move (item, 0, 50.0);
 	g_object_unref (item);
 	g_assert (G_OBJECT (item)->ref_count == 1);
@@ -872,7 +879,7 @@ main (int argc, char* argv[])
 
 	//dia_canvas_item_rotate (item, 154.0);
 	dia_canvas_item_move (item, 100.0, 100.0);
-	g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
+	//g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
 	g_signal_connect (item, "notify::parent", G_CALLBACK (visible_cb), NULL);
 	g_object_unref (item);
 	g_assert (G_OBJECT (item)->ref_count == 1);
@@ -887,7 +894,7 @@ main (int argc, char* argv[])
 
 	dia_canvas_item_move (item, 100.0, 100.0);
 	dia_canvas_item_request_update (item);
-	g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
+	//g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
 	g_object_unref (item);
 	g_assert (G_OBJECT (item)->ref_count == 1);
 	/* Draw an ellipse */
@@ -932,7 +939,7 @@ main (int argc, char* argv[])
 		      "markup", TRUE,
 		      NULL);
 	dia_canvas_item_move (item, 10.0, 10.0);
-	g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
+	//g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
 	g_object_unref (item);
 	g_assert (G_OBJECT (item)->ref_count == 1);
 
@@ -952,7 +959,7 @@ main (int argc, char* argv[])
 		      "multiline", 0,
 		      NULL);
 	dia_canvas_item_move (item, 10.0, 150.0);
-	g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
+	//g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
 	g_object_unref (item);
 	g_assert (G_OBJECT (item)->ref_count == 1);
 
@@ -972,7 +979,7 @@ main (int argc, char* argv[])
 		      "multiline", 0,
 		      NULL);
 	dia_canvas_item_move (item, 10.0, 180.0);
-	g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
+	//g_signal_connect (item, "event", G_CALLBACK (canvas_item_event), NULL);
 	g_object_unref (item);
 	g_assert (G_OBJECT (item)->ref_count == 1);
 
