@@ -238,7 +238,8 @@ dia_canvas_item_class_init (DiaCanvasItemClass *klass)
 						 _("GList of handles"),
 					 _("List of handles, used internaly"),
 					 DIA_TYPE_CANVAS_ITEM_HANDLES,
-					 G_PARAM_READWRITE));
+					 G_PARAM_READABLE));
+					 //G_PARAM_READWRITE));
 
 	/* Callbacks */
 	klass->update = dia_real_canvas_item_update;
@@ -368,16 +369,12 @@ dia_canvas_item_dispose (GObject *object)
 	GList *l;
 	DiaCanvasItem *item = (DiaCanvasItem*) object;
 	
-#ifdef ENABLE_DEBUG
-	g_message (G_STRLOC": %s (%p)/%p", G_OBJECT_TYPE_NAME (object), object, item->parent);
-#endif
-
 	dia_canvas_item_ungrab (item);
 
+	//g_message (G_STRLOC": disposing %p", item);
+
 	if (item->parent)
 		dia_canvas_item_set_child_of (item, NULL);
-		//dia_canvas_groupable_remove_destruction (DIA_CANVAS_GROUPABLE (item->parent), item);
-
 
 #ifdef ENABLE_DEBUG
 	g_assert (item->parent == NULL);
@@ -407,9 +404,6 @@ dia_canvas_item_finalize (GObject *object)
 	GList *l;
 	DiaCanvasItem *item = (DiaCanvasItem*) object;
 
-#ifdef ENABLE_DEBUG
-	g_message (G_STRLOC": %s (%p)", G_OBJECT_TYPE_NAME (object), object);
-#endif
 	for (l = item->handles; l != NULL; l = l->next) {
 		g_object_unref (l->data);
 	}
@@ -461,19 +455,6 @@ dia_real_canvas_item_move (DiaCanvasItem *item, gdouble dx, gdouble dy,
 	item->affine[4] += dx;
 	item->affine[5] += dy;
 
-	/*
-	if (item->canvas && item->canvas->snap_to_grid) {
-		gdouble x, y;
-		x = item->affine[4];
-		y = item->affine[5];
-		dia_canvas_item_affine_point_i2w (item, &x, &y);
-		dia_canvas_snap_to_grid (item->canvas, &x, &y);
-		dia_canvas_item_affine_point_w2i (item, &x, &y);
-		item->affine[4] = x;
-		item->affine[5] = y;
-	}
-	*/
-
 	dia_canvas_item_update_handles_i2w (item);
 	dia_canvas_item_request_update (item);
 }
@@ -481,23 +462,14 @@ dia_real_canvas_item_move (DiaCanvasItem *item, gdouble dx, gdouble dy,
 static gboolean
 dia_real_canvas_item_connect (DiaCanvasItem *item, DiaHandle *handle)
 {
+	dia_handle_preserve_state (handle);
+
 	if (handle->connected_to != item) {
-		handle->connected_to = item;
+		if (handle->connected_to)
+			dia_canvas_item_disconnect (handle->connected_to, handle);
+		handle->connected_to = g_object_ref (item);
 		item->connected_handles = g_list_append (item->connected_handles,
 							 handle);
-		/* When undoing, we should disconnect this connection... */
-		if (item->canvas)
-			dia_canvas_preserve_property (item->canvas,
-						      G_OBJECT (handle),
-						      "disconnect");
-	} else {
-		/* If we move the handle around on the same item, it will only
-		 * do a "connect" in order to set up a new set of constraints.
-		 */
-		if (item->canvas)
-			dia_canvas_preserve_property_last (item->canvas,
-							   G_OBJECT (handle),
-							   "connect");
 	}
 	return TRUE;
 }
@@ -507,14 +479,13 @@ dia_real_canvas_item_disconnect (DiaCanvasItem *item, DiaHandle *handle)
 {
 	g_assert (handle->connected_to == item);
 
-	/* When undoing, we should do a "connect"... */
-	if (item->canvas)
-		dia_canvas_preserve_property_last (item->canvas,
-						   G_OBJECT (handle),
-						   "connect");
+	dia_handle_preserve_state (handle);
 
 	dia_handle_remove_all_constraints (handle);
 
+	if (handle->connected_to)
+		g_object_unref (handle->connected_to);
+
 	handle->connected_to = NULL;
 	item->connected_handles = g_list_remove (item->connected_handles, handle);
 
@@ -592,12 +563,14 @@ dia_canvas_item_set_parent (DiaCanvasItem *item, DiaCanvasItem *new_parent)
 
 	if (item->parent == new_parent)
 		return;
+
 	g_object_ref (item);
 	if (item->parent != NULL)
 		dia_canvas_groupable_remove (DIA_CANVAS_GROUPABLE (item->parent), item);
+
 	if (new_parent && item->parent == NULL) {
 		g_assert (DIA_IS_CANVAS_GROUPABLE (new_parent));
-		//g_message (__FUNCTION__": setting new parent.");
+		//g_message (G_STRLOC": setting new parent.");
 		dia_canvas_groupable_add (DIA_CANVAS_GROUPABLE (new_parent), item);
 	}
 	g_object_unref (item);
@@ -628,11 +601,16 @@ dia_canvas_item_set_child_of (DiaCanvasItem *item, DiaCanvasItem *new_parent)
 	if (new_parent)
 		g_return_if_fail (DIA_IS_CANVAS_GROUPABLE (new_parent));
 
-	//g_message(G_STRLOC ": Setting new parent to %p (%p)", new_parent, new_parent ? new_parent->canvas : NULL);
+	//g_message(G_STRLOC": Setting new parent to %p (%p)", new_parent, new_parent ? new_parent->canvas : NULL);
 
 	/* Notify is filtered, so only one parent notification will be emited */
 	g_object_freeze_notify (G_OBJECT(item));
 
+	if ((!new_parent) || item->canvas != new_parent->canvas) {
+		/* Ensure the handles are disconnected */
+		dia_canvas_item_disconnect_handles (item);
+	}
+
 	/* Disconnect the parent when needed, we store this in the
 	 * current canvas */
 	if (item->parent && item->parent != new_parent) {
@@ -646,7 +624,7 @@ dia_canvas_item_set_child_of (DiaCanvasItem *item, DiaCanvasItem *new_parent)
 	/* Now update the canvas, so we can save undo information */
 	if ((!new_parent) || item->canvas != new_parent->canvas) {
 		/* Ensure the handles are disconnected */
-		dia_canvas_item_disconnect_handles (item);
+		//dia_canvas_item_disconnect_handles (item);
 
 		if (item->canvas)
 			g_object_weak_unref (G_OBJECT (item->canvas),
@@ -656,11 +634,12 @@ dia_canvas_item_set_child_of (DiaCanvasItem *item, DiaCanvasItem *new_parent)
 		if (item->canvas)
 			g_object_weak_ref (G_OBJECT (item->canvas),
 					   canvas_destroyed, item);
+		//g_message(G_STRLOC": resetting canvas to %p", item->canvas);
 	}
 
 	/* Set new parent, saving the old value on the canvas' undo stack */
 	if (new_parent && item->parent != new_parent) {
-		dia_canvas_item_preserve_property (item, "parent");
+		//dia_canvas_item_preserve_property (item, "parent");
 		item->parent = new_parent;
 		g_object_add_weak_pointer (G_OBJECT (item->parent),
 					   (gpointer) &item->parent);
@@ -724,14 +703,14 @@ dia_canvas_item_request_update (DiaCanvasItem *item)
 
 	dia_canvas_request_update (item->canvas);
 
-	//g_message (__FUNCTION__": doing item: %p (%d)", i, i ? i->flags & DIA_NEED_UPDATE : -1);
+	//g_message (G_STRLOC": doing item: %p (%d)", i, i ? i->flags & DIA_NEED_UPDATE : -1);
 
 	/* Update the whole tree */
 	while (i && !DIA_CANVAS_ITEM_NEED_UPDATE (i)) {
-		//g_message(__FUNCTION__": need update: %p", i);
+		//g_message(G_STRLOC": need update: %p", i);
 		DIA_SET_FLAGS (i, DIA_NEED_UPDATE);
 		i = (DiaCanvasItem*) i->parent;
-		//g_message (__FUNCTION__": doing parent: %p (%d)", i, i ? i->flags & DIA_NEED_UPDATE : -1);
+		//g_message (G_STRLOC": doing parent: %p (%d)", i, i ? i->flags & DIA_NEED_UPDATE : -1);
 	}
 
 	/* update child objects: */
@@ -1602,3 +1581,4 @@ dia_canvas_item_preserve_property (DiaCanvasItem *item,
 					      G_OBJECT (item), property_name);
 }
 
+
