@@ -23,6 +23,7 @@
 #include <libart_lgpl/art_affine.h>
 #include <pango/pangoft2.h>
 #include "dia-canvas.h"
+#include "dia-undo.h"
 #include "dia-canvas-group.h"
 #include "dia-canvas-i18n.h"
 #include "diamarshal.h"
@@ -56,26 +57,6 @@ enum {
 	PROP_PANGO_LAYOUT
 };
 
-typedef struct {
-	GObject *object;
-	const gchar *property_name;
-	GValue value;
-} UndoData;
-
-typedef struct  {
-	gchar *comment;
-	GArray *undo_data; /* List of UndoData. */
-} UndoEntry;
-
-static UndoEntry* undo_entry_new	(void);
-static void undo_entry_free		(UndoEntry *undo_entry);
-static void undo_entry_append		(UndoEntry *undo_entry,
-					 GObject *object,
-					 GParamSpec *pspec,
-					 const GValue *value,
-					 gboolean last);
-static void undo_entry_apply		(UndoEntry *undo_entry);
-
 /* DiaCanvas static (private) functions. */
 static void dia_canvas_init		(DiaCanvas	*canvas);
 static void dia_canvas_class_init	(DiaCanvasClass	*klass);
@@ -309,10 +290,8 @@ dia_canvas_init (DiaCanvas *canvas)
 
 	canvas->allow_state_requests = TRUE;
 	canvas->allow_undo = FALSE;
-	canvas->in_undo = FALSE;
-	canvas->stack_depth = 10;
-	canvas->undo = NULL;
-	canvas->redo = NULL;
+	//canvas->in_undo = FALSE;
+	canvas->undo_manager = g_object_new(dia_undo_get_type(), NULL);
 }
 
 static void
@@ -436,25 +415,9 @@ dia_canvas_dispose (GObject *object)
 	canvas->allow_undo = FALSE;
 
 	/* Now get rid of undo and redo information. */
-	if (canvas->undo_entry) {
-		undo_entry_free (canvas->undo_entry);
-		canvas->undo_entry = NULL;
-	}
-
-	if (canvas->undo) {
-		GSList *l;
-		for (l = canvas->undo; l != NULL; l = l->next)
-			undo_entry_free (l->data);
-		g_slist_free (canvas->undo);
-		canvas->undo = NULL;
-	}
-
-	if (canvas->redo) {
-		GSList *l;
-		for (l = canvas->redo; l != NULL; l = l->next)
-			undo_entry_free (l->data);
-		g_slist_free (canvas->redo);
-		canvas->redo = NULL;
+	if (canvas->undo_manager) {
+		g_object_unref (canvas->undo_manager);
+		canvas->undo_manager = NULL;
 	}
 
 	/* Remove idle handler, we set the idle_id to -1, so it won't
@@ -464,6 +427,18 @@ dia_canvas_dispose (GObject *object)
 		canvas->idle_id = -1;
 	}
 
+	/* First dispose the canvas items. */
+	if (canvas->root) {
+		canvas->root->canvas = NULL;
+		g_object_unref (G_OBJECT (canvas->root));
+		canvas->root = NULL;
+	}
+
+	if (canvas->solver) {
+		g_object_unref (G_OBJECT (canvas->solver));
+		canvas->solver = NULL;
+	}
+
 	/* Do this one last, so we're sure no canvas item try to update. */
 #ifdef ENABLE_DEBUG
 	g_message (G_STRLOC": idle_id = %d", canvas->idle_id);
@@ -486,16 +461,16 @@ dia_canvas_finalize (GObject *object)
 #endif
 
 	/* First dispose the canvas items. */
-	if (canvas->root) {
+	/*if (canvas->root) {
 		canvas->root->canvas = NULL;
 		g_object_unref (G_OBJECT (canvas->root));
 		canvas->root = NULL;
-	}
+	}*/
 
-	if (canvas->solver) {
+	/*if (canvas->solver) {
 		g_object_unref (G_OBJECT (canvas->solver));
 		canvas->solver = NULL;
-	}
+	}*/
 
 
 	parent_class->finalize (object);
@@ -522,7 +497,7 @@ dia_real_canvas_update (DiaCanvas *canvas)
 
 	dia_canvas_item_update_now (canvas->root);
 
-	if (!canvas->static_extents)
+	if (!canvas->static_extents && canvas->root)
 		dia_canvas_set_extents (canvas, &canvas->root->bounds);
 }
 
@@ -611,7 +586,7 @@ dia_canvas_set_static_extents (DiaCanvas *canvas, gboolean stat)
 	g_return_if_fail (DIA_IS_CANVAS (canvas));
 
 	canvas->static_extents = stat;
-	if (!stat)
+	if (!stat && canvas->root)
 		dia_canvas_set_extents (canvas, &canvas->root->bounds);
 
 	g_object_notify (G_OBJECT (canvas), "static_extents");
@@ -936,6 +911,25 @@ dia_canvas_redraw_views (DiaCanvas *canvas)
 	g_signal_emit (canvas, canvas_signals[REDRAW_VIEW], 0);
 }
 
+DiaUndoManager*
+dia_canvas_get_undo_manager (DiaCanvas *canvas)
+{
+	g_return_if_fail (DIA_IS_CANVAS (canvas));
+
+	return canvas->undo_manager;
+}
+
+void
+dia_canvas_set_undo_manager (DiaCanvas *canvas, DiaUndoManager *undo_manager)
+{
+	g_return_if_fail (DIA_IS_CANVAS (canvas));
+	g_return_if_fail (DIA_IS_UNDO_MANAGER (undo_manager));
+
+	if (canvas->undo_manager)
+		g_object_unref (canvas->undo_manager);
+
+	canvas->undo_manager = g_object_ref (undo_manager);
+}
 
 /*
  * UNDO/REDO
@@ -945,30 +939,73 @@ dia_canvas_redraw_views (DiaCanvas *canvas)
 //#define UNDOINFO(...) g_message (G_STRLOC": " __VA_ARGS__);
 #define UNDOINFO(...)
 
-/**
- * verify_stack_depth:
- * @stack:
- * @depth: 
- *
- * Ensure that the undo stack is not deeper that @depth entries.
- **/
-static void
-verify_stack_depth (GSList *stack, guint depth)
+typedef struct _DiaUndoProperty DiaUndoProperty;
+
+struct _DiaUndoProperty
 {
-	if (!stack)
-		return;
-	while (g_slist_length (stack) > depth) {
-		GSList *last = g_slist_last (stack);
-		UNDOINFO("stack to deep (max %d allowed)", depth);
-		undo_entry_free (last->data);
-		g_slist_delete_link (stack, last);
-	}
+	DiaUndoAction entry;
+
+	GObject *object;
+	GParamSpec *pspec;
+	GValue value;
+	GValue redo_value;
+};
+
+void
+dia_undo_property_undo (DiaUndoAction *entry)
+{
+	DiaUndoProperty *p = (DiaUndoProperty*) entry;
+
+	UNDOINFO("Applying property '%s'", p->pspec->name);
+
+	// Save value for redo purposes
+	g_object_get_property (p->object, p->pspec->name, &p->redo_value);
+
+	g_object_set_property (p->object, p->pspec->name, &p->value);
 }
 
-static void
-emit_undo(DiaCanvas *canvas)
+void
+dia_undo_property_redo (DiaUndoAction *entry)
 {
-	g_signal_emit (canvas, canvas_signals[UNDO], 0);
+	DiaUndoProperty *p = (DiaUndoProperty*) entry;
+
+	UNDOINFO("Applying property '%s'", p->pspec->name);
+	g_object_set_property (p->object, p->pspec->name, &p->redo_value);
+}
+
+void
+dia_undo_property_destroy (gpointer entry)
+{
+	DiaUndoProperty *p = (DiaUndoProperty*) entry;
+
+	g_object_unref (p->object);
+	g_value_unset (&p->value);
+	g_value_unset (&p->redo_value);
+	g_param_spec_unref (p->pspec);
+}
+
+DiaUndoAction*
+dia_undo_property_new (GObject *object, GParamSpec *pspec, const GValue *value)
+{
+	DiaUndoProperty *p;
+
+	g_assert (G_IS_OBJECT (object));
+
+	p = (DiaUndoProperty*) dia_undo_action_new (sizeof (DiaUndoProperty),
+						dia_undo_property_undo,
+						dia_undo_property_redo,
+						dia_undo_property_destroy);
+
+	//g_message (G_STRLOC": saving property %s %p", pspec->name, object);
+
+	p->object = g_object_ref (object);
+	p->pspec = g_param_spec_ref (pspec);
+
+	g_value_init (&p->value, G_PARAM_SPEC_VALUE_TYPE (pspec));
+	g_value_init (&p->redo_value, G_PARAM_SPEC_VALUE_TYPE (pspec));
+	g_value_copy (value, &p->value);
+
+	return (DiaUndoAction*) p;
 }
 
 static void
@@ -980,20 +1017,12 @@ preserve (DiaCanvas *canvas, GObject *object,
 	if (!canvas->allow_undo)
 		return;
 
-	if (canvas->undo_entry == NULL) {
-		canvas->undo_entry = undo_entry_new ();
-		new = TRUE;
-	}
+	if (!canvas->undo_manager)
+		return;
 
-	/* Once the user starts adding new undo information, the redo stack is
-	 * considered garbage. */
-	if (!canvas->in_undo)
-		dia_canvas_clear_redo (canvas);
-
-	undo_entry_append (canvas->undo_entry, object, pspec, value, last);
-
-	if (new)
-		emit_undo(canvas);
+	if (dia_undo_manager_in_transaction (canvas->undo_manager))
+		dia_undo_manager_add_undo_action (canvas->undo_manager,
+				dia_undo_property_new (object, pspec, value));
 }
 
 /**
@@ -1112,19 +1141,25 @@ dia_canvas_push_undo (DiaCanvas *canvas, const char* optional_comment)
 {
 	g_return_if_fail (DIA_IS_CANVAS (canvas));
 
-	if (!canvas->allow_undo)
+	g_warning (G_STRLOC"This function is depricated.");
+
+	return;
+#if 0
+	if (!canvas->allow_undo || !canvas->undo_manager)
 		return;
 
+	dia_undo_manager_commit_transaction (canvas->undo_manager);
+
 	if (canvas->undo_entry &&
 	    (((UndoEntry*) canvas->undo_entry)->undo_data->len > 0)) { 
 		canvas->undo = g_slist_prepend (canvas->undo,
 						canvas->undo_entry);
-		verify_stack_depth (canvas->undo, canvas->stack_depth);
 		UNDOINFO("stack pushed %p %p (depth = %d)", canvas->undo_entry, canvas->undo->data, g_slist_length(canvas->undo));
 		canvas->undo_entry = NULL;
 		/* undo signal is emited by preserve() as soon as
 		   canvas->undo_entry is used. */
 	}
+#endif
 }
 	
 
@@ -1138,10 +1173,10 @@ dia_canvas_push_undo (DiaCanvas *canvas, const char* optional_comment)
 void
 dia_canvas_pop_undo (DiaCanvas *canvas)
 {
-	UndoEntry *undo_entry = NULL;
+	//UndoEntry *undo_entry = NULL;
 
 	g_return_if_fail (DIA_IS_CANVAS (canvas));
-
+#if 0
 	if (canvas->undo_entry && ((UndoEntry*)canvas->undo_entry)->undo_data->len > 0) {
 		UNDOINFO("from undo_entry");
 		undo_entry = canvas->undo_entry;
@@ -1167,13 +1202,11 @@ dia_canvas_pop_undo (DiaCanvas *canvas)
 
 	if (canvas->undo_entry && ((UndoEntry*) canvas->undo_entry)->undo_data->len) {
 		canvas->redo = g_slist_prepend (canvas->redo, canvas->undo_entry);
-		verify_stack_depth (canvas->redo, canvas->stack_depth);
 		canvas->undo_entry = NULL;
 	}
 
 	undo_entry_free (undo_entry);
-
-	emit_undo(canvas);
+#endif
 }
 
 /**
@@ -1185,8 +1218,10 @@ dia_canvas_pop_undo (DiaCanvas *canvas)
 void
 dia_canvas_pop_redo (DiaCanvas *canvas)
 {
-	UndoEntry *redo_entry;
+	//UndoEntry *redo_entry;
 
+	g_warning (G_STRLOC"This function is depricated.");
+#if 0
 	UNDOINFO();
 	g_return_if_fail (DIA_IS_CANVAS (canvas));
 
@@ -1210,13 +1245,11 @@ dia_canvas_pop_redo (DiaCanvas *canvas)
 
 	if (canvas->undo_entry && ((UndoEntry*) canvas->undo_entry)->undo_data->len) {
 		canvas->undo = g_slist_prepend (canvas->undo, canvas->undo_entry);
-		verify_stack_depth (canvas->undo, canvas->stack_depth);
 		canvas->undo_entry = NULL;
 	}
 
 	undo_entry_free (redo_entry);
-
-	emit_undo(canvas);
+#endif
 }
 
 /**
@@ -1230,18 +1263,7 @@ dia_canvas_clear_undo (DiaCanvas *canvas)
 {
 	g_return_if_fail (DIA_IS_CANVAS (canvas));
 
-	if (canvas->undo) {
-		GSList *l;
-		for (l = canvas->undo; l != NULL; l = l->next)
-			undo_entry_free (l->data);
-		g_slist_free (canvas->undo);
-		canvas->undo = NULL;
-		emit_undo(canvas);
-	}
-	if (canvas->undo_entry) {
-		undo_entry_free (canvas->undo_entry);
-		canvas->undo_entry = NULL;
-	}
+	g_warning (G_STRLOC"This function is depricated.");
 }
 
 /**
@@ -1256,14 +1278,7 @@ dia_canvas_clear_redo (DiaCanvas *canvas)
 {
 	g_return_if_fail (DIA_IS_CANVAS (canvas));
 
-	if (canvas->redo) {
-		GSList *l;
-		for (l = canvas->redo; l != NULL; l = l->next)
-			undo_entry_free (l->data);
-		g_slist_free (canvas->redo);
-		canvas->redo = NULL;
-		emit_undo(canvas);
-	}
+	g_warning (G_STRLOC"This function is depricated.");
 }
 
 /**
@@ -1280,8 +1295,8 @@ dia_canvas_get_undo_depth (DiaCanvas *canvas)
 {
 	g_return_val_if_fail (DIA_IS_CANVAS (canvas), 0);
 
-	return (canvas->undo ? g_slist_length (canvas->undo) : 0)
-		+ (canvas->undo_entry && ((UndoEntry*)canvas->undo_entry)->undo_data->len ? 1 : 0);
+	g_warning (G_STRLOC"This function is depricated.");
+	return 0;
 }
 
 /**
@@ -1297,7 +1312,8 @@ dia_canvas_get_redo_depth (DiaCanvas *canvas)
 {
 	g_return_val_if_fail (DIA_IS_CANVAS (canvas), 0);
 
-	return canvas->redo ? g_slist_length (canvas->redo) : 0;
+	g_warning (G_STRLOC"This function is depricated.");
+	return 0;
 }
 
 /**
@@ -1312,10 +1328,7 @@ dia_canvas_set_undo_stack_depth (DiaCanvas *canvas, guint depth)
 {
 	g_return_if_fail (DIA_IS_CANVAS (canvas));
 
-	canvas->stack_depth = depth;
-
-	verify_stack_depth (canvas->undo, depth);
-	verify_stack_depth (canvas->redo, depth);
+	g_warning (G_STRLOC"This function is depricated.");
 }
 
 /**
@@ -1331,101 +1344,7 @@ dia_canvas_get_undo_stack_depth (DiaCanvas *canvas)
 {
 	g_return_val_if_fail (DIA_IS_CANVAS (canvas), 0);
 
-	return canvas->stack_depth;
-}
-
-/*
- * UndoEntry
- */
-
-/* Create a new undo entry. An undo entry contains a collection of actions that
- * have to be undone in one sweep.
- */
-static UndoEntry*
-undo_entry_new	(void)
-{
-	UndoEntry *entry;
-
-	entry = g_new (UndoEntry, 1);
-
-	entry->comment = NULL;
-	entry->undo_data = g_array_new (FALSE, FALSE, sizeof (UndoData));
-
-	return entry;
-}
-
-static void
-undo_entry_free (UndoEntry *entry)
-{
-	guint i;
-
-	UNDOINFO();
-	for (i = 0; i < entry->undo_data->len; i++) {
-		UndoData data;
-		data = g_array_index (entry->undo_data, UndoData, i);
-		UNDOINFO(G_STRLOC": %p with refcount %d", data.object, data.object->ref_count);
-		g_object_unref (data.object);
-		g_value_unset (&data.value);
-	}
-	g_array_free (entry->undo_data, TRUE);
-	if (entry->comment)
-		g_free (entry->comment);
-	g_free (entry);
-}
-
-/**
- * undo_entry_append:
- * @entry: 
- * @object: is referenced.
- * @property_name: property name used to retrieve the GParamSpec.
- * @value:  Value is copied.
- * @last: 
- *
- **/
-static void
-undo_entry_append (UndoEntry *entry, GObject *object,
-		   GParamSpec *pspec, const GValue *value, gboolean last)
-{
-	UndoData data = { NULL, NULL, { 0, } };
-	guint i;
-	const gchar *property_name = g_param_spec_get_name (pspec);
-
-	/* Check if the object/property combination is not already in entry. */
-	for (i = 0; i < entry->undo_data->len; i++) {
-		UndoData ud = g_array_index (entry->undo_data, UndoData, i);
-		if ((ud.object == object)
-		    && (strcmp (ud.property_name, property_name) == 0))
-			return;
-	}
-
-	data.object = object;
-	g_object_ref (object);
-	data.property_name = property_name;
-	g_value_init (&data.value, G_VALUE_TYPE (value));
-	g_value_copy (value, &data.value);
-	if (last)
-		g_array_prepend_val (entry->undo_data, data);
-	else
-		g_array_append_val (entry->undo_data, data);
-
-	UNDOINFO("Added property %p '%s' (#%d)", object,
-		 data.property_name, entry->undo_data->len);
-}
-
-static void
-undo_entry_apply (UndoEntry *entry)
-{
-	guint i;
-
-	g_return_if_fail (entry != NULL);
-	g_return_if_fail (entry->undo_data->len > 0);
-
-	for (i = entry->undo_data->len; i > 0; i--) {
-		UndoData data = g_array_index (entry->undo_data, UndoData, i - 1);
-
-		UNDOINFO("Applying property '%s' %x (%d of %d)", data.property_name, (int) data.value.data[0].v_pointer, i, entry->undo_data->len);
-		g_object_set_property (data.object,
-				       data.property_name, &data.value);
-	}
+	g_warning (G_STRLOC"This function is depricated.");
+	return 0;
 }
 
