@@ -140,6 +140,7 @@ dia_handle_class_init (DiaHandleClass *klass)
 						DIA_TYPE_STRENGTH,
 						DIA_STRENGTH_WEAK,
 						G_PARAM_READWRITE));
+	/*
 	g_object_class_install_property (object_class,
 					 PROP_CONNECT,
 					 g_param_spec_object ("connect",
@@ -154,6 +155,7 @@ dia_handle_class_init (DiaHandleClass *klass)
 						_(""),
 						DIA_TYPE_CANVAS_ITEM,
 						G_PARAM_READWRITE));
+	*/
 	g_object_class_install_property (object_class,
 					 PROP_CONNECTED_TO,
 					 g_param_spec_object ("connected_to",
@@ -386,8 +388,13 @@ dia_handle_dispose (GObject *object)
 {
 	DiaHandle *handle = (DiaHandle*) object;
 
-	//g_message (__FUNCTION__": %x DISPOSING!!!", (int) object);
+	//g_message (G_STRLOC": %x DISPOSING!!!", (int) object);
 	dia_handle_remove_all_constraints (handle);
+
+	if (handle->connected_to) {
+		g_object_unref (handle->connected_to);
+		handle->connected_to = NULL;
+	}
 }
 
 static void
@@ -768,7 +775,6 @@ dia_handle_add_constraint (DiaHandle *handle, DiaConstraint *c)
 	if (!handle->owner->canvas)
 		return;
 
-	/* TODO: Preserve property constraints */
 	g_object_ref (c);
 	handle->constraints = g_slist_prepend (handle->constraints, c);
 	dia_canvas_add_constraint (handle->owner->canvas, c);
@@ -894,7 +900,6 @@ dia_handle_remove_constraint (DiaHandle *handle, DiaConstraint *c)
 	g_return_if_fail (g_slist_find (handle->constraints, c));
 	g_assert (handle->owner->canvas != NULL);
 
-	/* TODO: Preserve property constraints */
 	dia_canvas_remove_constraint (handle->owner->canvas, c);
 	handle->constraints = g_slist_remove (handle->constraints, c);
 	g_object_unref (c);
@@ -918,3 +923,203 @@ dia_handle_remove_all_constraints (DiaHandle *handle)
 	}
 }
 
+/*
+ * UNDO/REDO
+ */
+
+static GSList*
+dia_g_slist_deep_copy (GSList *l)
+{
+	GSList *t;
+	GSList *newl;
+
+	if (!l) return NULL;
+
+	newl = g_slist_copy (l);
+	for (t = newl; t != NULL; t = g_slist_next (t))
+		g_object_ref (t->data);
+
+	return newl;
+}
+
+static void
+dia_g_slist_deep_free (GSList *l)
+{
+	GSList *t;
+
+	for (t = l; t != NULL; t = g_slist_next (t))
+		g_object_unref (t->data);
+	g_slist_free (l);
+}
+
+typedef struct
+{
+	DiaUndoAction entry;
+
+	DiaHandle *handle;
+
+	/* undo data */
+	DiaPoint pos_i;
+	DiaCanvasItem *connected_to;
+	GSList *constraints;
+
+	/* redo data */
+	DiaPoint redo_pos_i;
+	DiaCanvasItem *redo_connected_to;
+	GSList *redo_constraints;
+} DiaUndoConnect;
+
+static void
+dia_undo_connect_undo (DiaUndoAction *entry)
+{
+	DiaUndoConnect *c = (DiaUndoConnect*) entry;
+	DiaHandle *handle = c->handle;
+	GSList *l;
+
+	//g_message (G_STRLOC);
+
+	c->redo_pos_i.x = dia_variable_get_value (handle->pos_i.x);
+	c->redo_pos_i.y = dia_variable_get_value (handle->pos_i.y);
+	dia_handle_set_pos_i (handle, c->pos_i.x, c->pos_i.y);
+
+	if (handle->connected_to) {
+		if (!c->redo_connected_to && handle->connected_to)
+			c->redo_connected_to = g_object_ref (handle->connected_to);
+			//c->redo_connected_to = handle->connected_to; //g_object_ref (handle->connected_to);
+		handle->connected_to->connected_handles = g_list_remove (handle->connected_to->connected_handles, handle);
+		//g_object_unref (handle->connected_to);
+	}
+	//handle->connected_to = c->connected_to; //g_object_ref (c->connected_to);
+	handle->connected_to = g_object_ref (c->connected_to);
+	//g_message(G_STRLOC": handle conn. to %p", handle->connected_to);
+
+	if (handle->connected_to) {
+		handle->connected_to->connected_handles = g_list_append (handle->connected_to->connected_handles, handle);
+	}
+	//g_message (G_STRLOC": handle set");
+	
+
+	if (handle->constraints) {
+		if (!c->redo_constraints)
+			c->redo_constraints = dia_g_slist_deep_copy (handle->constraints);
+		dia_handle_remove_all_constraints (handle);
+	}
+	handle->constraints = dia_g_slist_deep_copy (c->constraints);
+
+	if (handle->owner && handle->owner->canvas)
+		for (l = handle->constraints; l != NULL; l = g_slist_next (l)) {
+			//g_message (G_STRLOC": adding constraint %p", l->data);
+			dia_canvas_add_constraint (handle->owner->canvas,
+						   l->data);
+		}
+}
+
+static void
+dia_undo_connect_redo (DiaUndoAction *entry)
+{
+	DiaUndoConnect *c = (DiaUndoConnect*) entry;
+	DiaHandle *handle = c->handle;
+	GSList *l;
+
+	dia_handle_set_pos_i (handle, c->redo_pos_i.x, c->redo_pos_i.y);
+
+	if (handle->connected_to) {
+		handle->connected_to->connected_handles = g_list_remove (handle->connected_to->connected_handles, handle);
+		//g_object_unref (handle->connected_to);
+	}
+	handle->connected_to = g_object_ref (c->redo_connected_to);
+	//handle->connected_to = c->redo_connected_to; //g_object_ref (c->redo_connected_to);
+	if (handle->connected_to) {
+		handle->connected_to->connected_handles =
+			g_list_append (handle->connected_to->connected_handles, handle);
+		//g_object_ref (handle);
+	}
+	if (handle->constraints)
+		dia_handle_remove_all_constraints (handle);
+
+	handle->constraints = dia_g_slist_deep_copy (c->redo_constraints);
+
+	if (handle->owner && handle->owner->canvas)
+		for (l = handle->constraints; l != NULL; l = g_slist_next (l))
+			dia_canvas_add_constraint (handle->owner->canvas,
+						   l->data);
+}
+
+static void
+dia_undo_connect_destroy (gpointer entry)
+{
+	DiaUndoConnect *c = (DiaUndoConnect*) entry;
+
+	g_object_unref (c->handle);
+
+	if (c->connected_to) {
+		g_object_unref (c->connected_to);
+		c->connected_to = NULL;
+	}
+	if (c->constraints) {
+		dia_g_slist_deep_free (c->constraints);
+		c->constraints = NULL;
+	}
+	if (c->redo_connected_to) {
+		g_object_unref (c->redo_connected_to);
+		c->redo_connected_to = NULL;
+	}
+	if (c->redo_constraints) {
+		dia_g_slist_deep_free (c->redo_constraints);
+		c->redo_constraints = NULL;
+	}
+}
+
+static DiaUndoAction*
+dia_undo_connect_new (DiaHandle *handle)
+{
+	GSList *l;
+	DiaUndoConnect *c = (DiaUndoConnect*)
+			dia_undo_action_new (sizeof (DiaUndoConnect),
+					    dia_undo_connect_undo,
+					    dia_undo_connect_redo,
+					    dia_undo_connect_destroy);
+
+	//g_message(G_STRLOC": saving handle %p", handle);
+
+	c->handle = g_object_ref (handle);
+
+	c->pos_i.x = dia_variable_get_value (handle->pos_i.x);
+	c->pos_i.y = dia_variable_get_value (handle->pos_i.y);
+
+	if (handle->connected_to)
+		c->connected_to = g_object_ref (handle->connected_to);
+		//c->connected_to = handle->connected_to; //g_object_ref (handle->connected_to);
+
+	c->constraints = dia_g_slist_deep_copy (handle->constraints);
+
+	//g_message(G_STRLOC": new DiaUndoConnect: %p, (%f, %f), %p, %p",
+	//		handle, c->pos_i.x, c->pos_i.y, c->connected_to, c->constraints);
+
+	return (DiaUndoAction*) c;
+}
+
+/**
+ * dia_handle_preserve_state:
+ * @handle: 
+ *
+ * Save the state of the handle to the DiaUndoManager connected to the
+ * canvas. The state consists of the position and a possible item the handle
+ * is connected to.
+ **/
+void
+dia_handle_preserve_state (DiaHandle *handle)
+{
+	DiaUndoManager *undo_manager;
+
+	g_return_if_fail (DIA_IS_HANDLE (handle));
+
+	if (!handle->owner || !handle->owner->canvas)
+		return;
+
+	undo_manager = dia_canvas_get_undo_manager (handle->owner->canvas);
+
+	dia_undo_manager_add_undo_action (undo_manager, dia_undo_connect_new (handle));
+	dia_canvas_item_preserve_property(handle->owner, "parent");
+}
+
