@@ -18,6 +18,7 @@
  */
 
 #include "dia-shape-art.h"
+#include "dia-shape-view-info.h"
 #include <math.h>
 #include <stdlib.h>
 #include <pango/pangoft2.h>
@@ -105,7 +106,7 @@ svp_dup (ArtSVP* svp)
 /*
  * Path
  */
-static void path_free (DiaShapeViewInfo *view_info);
+static void path_free (DiaCanvasViewItem *item, DiaShapeViewInfo *view_info);
 
 #define PERTURBATION 2e-3
 #define ELLIPSE 1e-3
@@ -246,7 +247,7 @@ create_svps_from_vpath (ArtVpath *vpath, gdouble line_width,
 
 	g_free (path);
 
-	/* The srtoke has a line wirdth, so it is probably the */
+	/* The stroke has a line width, so it is probably the */
 
 	if (stroke_svp && fill_svp) {
 		ArtUta *uta1, *uta2, *uta3;
@@ -271,21 +272,22 @@ typedef struct {
 } PathHelper;
 
 static void
-reset_path_helper (PathHelper *helper, GnomeCanvasItem *item)
+reset_path_helper (PathHelper *helper, DiaCanvasViewItem *item)
 {
 	ArtUta *repaint_uta;
+	GnomeCanvasItem *gitem = (GnomeCanvasItem*) item;
 
 	if (!helper)
 		return;
 
-	if (item && item->canvas && GTK_WIDGET_DRAWABLE (item->canvas)) {
+	if (item && gitem->canvas && GTK_WIDGET_DRAWABLE (gitem->canvas)) {
 		if (helper->stroke_svp) {
 			repaint_uta = art_uta_from_svp (helper->stroke_svp);
-			gnome_canvas_request_redraw_uta (item->canvas, repaint_uta);
+			gnome_canvas_request_redraw_uta (gitem->canvas, repaint_uta);
 		}
 		if (helper->fill_svp) {
 			repaint_uta = art_uta_from_svp (helper->fill_svp);
-			gnome_canvas_request_redraw_uta (item->canvas, repaint_uta);
+			gnome_canvas_request_redraw_uta (gitem->canvas, repaint_uta);
 		}
 	}
 	if (helper->stroke_svp) {
@@ -319,7 +321,7 @@ path_update (DiaShapePath *shape, DiaCanvasViewItem *item,
 	if (!view_info->data)
 		helper = view_info->data = g_new0 (PathHelper, 1);
 	else {
-		GnomeCanvasItem *item = view_info->key;
+		//GnomeCanvasItem *item = view_info->key;
 		helper = view_info->data;
 		reset_path_helper (helper, item);
 	}
@@ -394,14 +396,12 @@ path_render (DiaShape *shape, DiaCanvasViewItem *item,
  * This function is used by paths, beziers and ellipses.
  */
 static void
-path_free (DiaShapeViewInfo *view_info)
+path_free (DiaCanvasViewItem *item, DiaShapeViewInfo *view_info)
 {
-	GnomeCanvasItem *item;
 	PathHelper *helper;
 
 	g_assert (view_info != NULL);
 
-	item = view_info->key;
 	helper = view_info->data;
 	view_info->data = NULL;
 
@@ -435,7 +435,7 @@ bezier_update (DiaShapeBezier *shape, DiaCanvasViewItem *item,
 	if (!view_info->data)
 		helper = view_info->data = g_new0 (PathHelper, 1);
 	else {
-		GnomeCanvasItem *item = view_info->key;
+		//GnomeCanvasItem *item = view_info->key;
 		helper = view_info->data;
 		reset_path_helper (helper, item);
 	}
@@ -511,7 +511,7 @@ ellipse_update (DiaShapeEllipse *shape, DiaCanvasViewItem *item,
 	if (!view_info->data)
 		helper = view_info->data = g_new0 (PathHelper, 1);
 	else {
-		GnomeCanvasItem *item = view_info->key;
+		//GnomeCanvasItem *item = view_info->key;
 		helper = view_info->data;
 		reset_path_helper (helper, item);
 	}
@@ -571,7 +571,7 @@ typedef struct {
 	GSList *selection_box; /* List of ArtSVP's */
 } TextHelper;
 
-static void text_free (DiaShapeViewInfo *view_info);
+static void text_free (DiaCanvasViewItem *item, DiaShapeViewInfo *view_info);
 
 #if 0
 /* TODO: Remove this function, create a default function instead! */
@@ -812,7 +812,7 @@ text_render (DiaShape *shape, DiaCanvasViewItem *item,
 }
 
 static void
-text_free (DiaShapeViewInfo *view_info)
+text_free (DiaCanvasViewItem *item, DiaShapeViewInfo *view_info)
 {
 	TextHelper *helper = (TextHelper*) view_info->data;
 	g_free (helper->bitmap.buffer);
@@ -829,7 +829,7 @@ typedef struct {
 	GdkPixbuf *pixbuf;
 } ImageHelper;
 
-static void image_free (DiaShapeViewInfo *view_info);
+static void image_free (DiaCanvasViewItem *item, DiaShapeViewInfo *view_info);
 
 static inline ArtSVP*
 image_update (DiaShapeImage *shape, DiaCanvasViewItem *item,
@@ -949,12 +949,12 @@ image_render (DiaShape *shape, DiaCanvasViewItem *item,
 }
 
 static void
-image_free (DiaShapeViewInfo *view_info)
+image_free (DiaCanvasViewItem *item, DiaShapeViewInfo *view_info)
 {
 	if (view_info->data) {
 		ImageHelper *helper = view_info->data;
-		GnomeCanvasItem *item = view_info->key;
-		gnome_canvas_request_redraw (item->canvas,
+		//GnomeCanvasItem *item = view_info->key;
+		gnome_canvas_request_redraw (GNOME_CANVAS_ITEM (item)->canvas,
 					     helper->bb.x0, helper->bb.y0,
 					     helper->bb.x1, helper->bb.y1);
 		if (helper->pixbuf)
@@ -991,20 +991,15 @@ dia_shape_art_update (DiaShape *shape, DiaCanvasViewItem *item,
 	
 	/* First, find the old shape-item-info (if none, create a new entry)
 	 */
-	view_info = dia_shape_get_view_info (shape, item);
+	view_info = dia_shape_view_info_get (item, shape);
 	
-	if (!view_info) {
-		//g_message (__FUNCTION__": Creating new view info for %s (%p)",
-		//	   g_type_name (item->item), item->item);
-		view_info = g_new0 (DiaShapeViewInfo, 1);
-		shape->view_info = g_list_append (shape->view_info, view_info);
-		view_info->key = (gpointer) item;
-		is_new = TRUE;
-	} else if (!dia_shape_need_update (shape)
-		   && !(GTK_OBJECT_FLAGS (item) & DIA_CANVAS_VIEW_ITEM_UPDATE_SHAPES))
-			return NULL;
+	g_assert (view_info != NULL);
+
+	//if (!dia_shape_need_update (shape)
+	//	   && !(GTK_OBJECT_FLAGS (item) & DIA_CANVAS_VIEW_ITEM_UPDATE_SHAPES))
+	//		return NULL;
 	
-	g_assert (DIA_IS_CANVAS_VIEW_ITEM (view_info->key));
+	//g_assert (DIA_IS_CANVAS_VIEW_ITEM (view_info->key));
 	
 	switch (shape->type) {
 	case DIA_SHAPE_PATH:
@@ -1049,12 +1044,10 @@ dia_shape_art_render (DiaShape *shape, DiaCanvasViewItem *item,
 
 	g_assert (DIA_IS_CANVAS_VIEW_ITEM (item));
 
-	view_info = dia_shape_get_view_info  (shape, item);
+	view_info = dia_shape_view_info_get (item, shape);
 	if (!view_info)
 		return;
 		
-	g_assert (DIA_IS_CANVAS_VIEW_ITEM (view_info->key));
-		
 	switch (shape->type) {
 	case DIA_SHAPE_PATH:
 	case DIA_SHAPE_BEZIER:
