@@ -1,4 +1,4 @@
-/* dia-shape.h
+/* dia-shape.c
  * Copyright (C) 2000  Arjan Molenaar
  *
  * This library is free software; you can redistribute it and/or
@@ -29,67 +29,14 @@ static void dia_shape_ellipse_free	(DiaShapeEllipse *ellipse);
 static void dia_shape_text_free		(DiaShapeText *text);
 static void dia_shape_image_free	(DiaShapeImage *image);
 
-/**
- * dia_shape_clean_view_info:
- * @shape: 
- * @key: 
- *
- * Find the DiaShapeViewInfo that belongs to @key. In most cases @key
- * is the #DiaCanvasViewItem object that draws the shape.
- *
- * Return value: 
- **/
-DiaShapeViewInfo*
-dia_shape_get_view_info (DiaShape *shape, gpointer key)
-{
-	GList *l;
-	for (l = shape->view_info; l != NULL; l = l->next) {
-		if (((DiaShapeViewInfo*) l->data)->key == key) {
-			//g_message ("Have view info");
-			return (DiaShapeViewInfo*) l->data;
-		}
-	}
-	return NULL;
-}
-
-/**
- * dia_shape_view_info_clear:
- * @view_info: 
- *
- * Empty the @view_info structure.
- **/
-void
-dia_shape_view_info_clear (DiaShapeViewInfo *view_info)
-{
-	if (view_info->free_func)
-		view_info->free_func (view_info);
-	view_info->free_func = NULL;
-	view_info->data = NULL;
-}
-
-/**
- * dia_shape_view_info_remove:
- * @shape: 
- * @view_info: 
- *
- * Remove @view_info from the shape. @view_info is also freed.
- **/
-void
-dia_shape_view_info_remove (DiaShape *shape, DiaShapeViewInfo *view_info)
-{
-	dia_shape_view_info_clear (view_info);
-	shape->view_info = g_list_remove (shape->view_info, view_info);
-	g_free (view_info);
-}
-
 /*
  * DiaShape -- updates
  */
 static inline void
 request_update (DiaShape *shape)
 {
-	if (!shape->update_cnt)
-		shape->update_cnt = g_list_length (shape->view_info);
+//	if (!shape->update_cnt)
+//		shape->update_cnt = g_list_length (shape->view_info);
 }
 
 void
@@ -103,7 +50,7 @@ dia_shape_need_update (DiaShape *shape)
 {
 	g_return_val_if_fail (shape != NULL, FALSE);
 
-	return (shape->update_cnt > 0 ? TRUE : FALSE);
+	return TRUE; (shape->update_cnt > 0 ? TRUE : FALSE);
 }
 
 void
@@ -237,17 +184,15 @@ dia_shape_new (DiaShapeType type)
 	shape->type = type;
 	shape->color = DIA_COLOR (0, 0, 0);
 	shape->update_cnt = 0;
+	shape->ref_cnt = 1;
 	shape->visibility = DIA_SHAPE_VISIBLE;
 
 	return shape;
 }
 
-void
-dia_shape_free (DiaShape *shape)
+static void
+dia_shape_destroy (DiaShape *shape)
 {
-	GList *l;
-	DiaShapeViewInfo *view_info;
-
 	g_return_if_fail (shape != NULL);
 
 	switch (shape->type) {
@@ -270,17 +215,36 @@ dia_shape_free (DiaShape *shape)
 		break;
 	}
 
-	/* Free the list of data added by DiaCanvasViewItem's */
-	for (l = shape->view_info; l != NULL; l = l->next) {
-		view_info = (DiaShapeViewInfo*) l->data;
-		if (view_info && view_info->free_func)
-			view_info->free_func (view_info);
-		g_free (view_info);
-	}
-	g_list_free (shape->view_info);
 	g_free (shape);
 }
 
+DiaShape*
+dia_shape_ref (DiaShape *shape)
+{
+	g_return_if_fail (shape != NULL);
+
+	shape->ref_cnt++;
+
+	return shape;
+}
+
+void
+dia_shape_unref (DiaShape *shape)
+{
+	g_return_if_fail (shape != NULL);
+
+	shape->ref_cnt--;
+
+	if (shape->ref_cnt == 0)
+		dia_shape_destroy (shape);
+}
+
+void
+dia_shape_free (DiaShape *shape)
+{
+	dia_shape_unref (shape);
+}
+
 /*
  * Generic attributes
  */
